<?php
/*
	Color lib by Yosko (http://www.yosko.net/)
	Licence: WTFPL except for the following
	source for HSL algorithm: http://www.easyrgb.com/index.php?X=MATH
*/

require_once ('color.lib.php');

//step size between 0 and 1
define("STEP_SIZE", (float)0.2);

if(isset($_GET['color'])) {
    $sourceColor = $_GET['color'];
} else {
    $sourceColor = "b7ff00";
}

$color = cleanHexColor($sourceColor);

$rgb = hex2rgb($color);
$hsl = rgb2hsl($rgb);

$samplesConstHue = variationAround($hsl, STEP_SIZE, "hue");
$samplesConstSat = variationAround($hsl, STEP_SIZE, "sat");
$samplesConstLum = variationAround($hsl, STEP_SIZE, "lum");

?><!DOCTYPE html>
<html lang="fr">
<head>
    <meta charset="utf-8">
    <title>HSL Color Sampler</title>
    <style>
    table { border-collapse: collapse; border: 2px solid black; text-align:center; }
    td, .preview { width:70px; height:50px; }
    th { border: 1px solid black; }
    .preview { display:table-cell; vertical-align:middle; text-align:center; color:#fff; font-weight: bold; }
    </style>
</head>
<body>
    <h1>HSL Color Sampler</h1>
    
    <h2>Source color</h2>
    <form method="get">
        <input type="text" name="color" value="<?php echo $sourceColor; ?>">
        <button type="send">
            Sample
        </button>
    </form>
    <div>6 digit hexadecimal number</div>
    <div>Configuration: <?php echo STEP_SIZE; ?> step variations</div>
    <div>Interpreted color : <span class="preview" style="background:#<?php echo $color; ?>;">#<?php echo $color; ?></span></div>
    <div>
        Note: the chosen color is supposed to appear in each (white text), but due to some rounded numbers, it may slightly differ (ex: 162417 instead of 162517).
    </div>
    
    <h2>Saturation/Luminance variation for a constant Hue</h2>
    <div>Hue : <?php echo round($hsl[0],2); ?></div>
    <table><?php
    $firsRow = array_keys(array_shift(array_values($samplesConstHue)));
    echo "<tr><th>sat \ lum</th>";
    for($i = 0; $i < count($firsRow); $i++) {
        echo "<th>".$firsRow[$i]."</th>";
    }
    echo "</tr>";
    foreach($samplesConstHue as $sat => $row) {
        $thisRow = (round($hsl[1],1) == round($sat,1));
        echo "\n\t\t<tr><th>".$sat."</th>";
        foreach($row as $lum => $value) {
            $thisLine = (round($hsl[2],1) == round($lum,1));
            $rgb = hsl2rgb($value);
            $html = "#".rgb2hex($rgb);
            if($thisRow && $thisLine)
                echo "<td class=\"preview\" style=\"background:".$html.";\">".$html."</td>";
            else
                echo "<td style=\"background:".$html.";\">".$html."</td>";
        }
        echo "</tr>";
    }
    ?>
    
    </table>
    
    <h2>Hue/Luminance variation for a constant Saturation</h2>
    <div>Saturation : <?php echo round($hsl[1],2); ?></div>
    <table><?php
    $firsRow = array_keys(array_shift(array_values($samplesConstSat)));
    echo "<tr><th>sat \ lum</th>";
    for($i = 0; $i < count($firsRow); $i++) {
        echo "<th>".$firsRow[$i]."</th>";
    }
    echo "</tr>";
    foreach($samplesConstSat as $hue => $row) {
        $thisRow = (round($hsl[0],1) == round($hue,1));
        echo "\n\t\t<tr><th>".$hue."</th>";
        foreach($row as $lum => $value) {
            $thisLine = (round($hsl[2],1) == round($lum,1));
            $rgb = hsl2rgb($value);
            $html = "#".rgb2hex($rgb);
            if($thisRow && $thisLine)
                echo "<td class=\"preview\" style=\"background:".$html.";\">".$html."</td>";
            else
                echo "<td style=\"background:".$html.";\">".$html."</td>";
        }
        echo "</tr>";
    }
    ?>
    
    </table>
    
    <h2>Hue/Saturation variation for a constant Luminance</h2>
    <div>Luminance : <?php echo round($hsl[2],2); ?></div>
    <table><?php
    $firsRow = array_keys(array_shift(array_values($samplesConstLum)));
    echo "<tr><th>sat \ lum</th>";
    for($i = 0; $i < count($firsRow); $i++) {
        echo "<th>".$firsRow[$i]."</th>";
    }
    echo "</tr>";
    foreach($samplesConstLum as $hue => $row) {
        $thisRow = (round($hsl[0],1) == round($hue,1));
        echo "\n\t\t<tr><th>".$hue."</th>";
        foreach($row as $sat => $value) {
            $thisLine = (round($hsl[1],1) == round($sat,1));
            $rgb = hsl2rgb($value);
            $html = "#".rgb2hex($rgb);
            if($thisRow && $thisLine)
                echo "<td class=\"preview\" style=\"background:".$html.";\">".$html."</td>";
            else
                echo "<td style=\"background:".$html.";\">".$html."</td>";
        }
        echo "</tr>";
    }
    ?>
    
    </table>
</body>